@@ -23,9 +23,6 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-static char     _config_gui_id[] =
-    "$Id: config_gui.c,v 1.3 2004/08/25 10:08:40 rogerms Exp $";
-
 
 #include "config_gui.h"
 
@@ -35,10 +32,8 @@ static char     _config_gui_id[] =
 
 #include <gtk/gtk.h>
 
-#include <libxfce4util/i18n.h>
+#include <libxfce4util/libxfce4util.h>
 #include <libxfcegui4/dialogs.h>
-#include <panel/plugins.h>
-#include <panel/xfce.h>
 
 
 #define COPYVAL(var, field)	((var)->field = field)
@@ -60,16 +55,8 @@ static char     _config_gui_id[] =
 #define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
   g_object_set_data (G_OBJECT (component), name, widget)
 
-#if 0
-GtkWidget      *create_OptionDialog (void)
-#else
-int CreateConfigGUI (GtkWidget * OptionDialog, struct gui_t *p_poGUI)
-#endif
+int CreateConfigGUI (GtkWidget * vbox1, struct gui_t *p_poGUI)
 {
-#if 0
-    GtkWidget      *OptionDialog;
-#endif
-    GtkWidget      *vbox1;
     GtkWidget      *wPB_About;
     GtkWidget      *alignment2;
     GtkWidget      *hbox3;
@@ -114,15 +101,6 @@ int CreateConfigGUI (GtkWidget * OptionDialog, struct gui_t *p_poGUI)
 
     tooltips = gtk_tooltips_new ();
 
-#if 0
-    OptionDialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_title (GTK_WINDOW (OptionDialog), _("OptionDialog"));
-#endif
-
-    vbox1 = gtk_vbox_new (FALSE, 0);
-    gtk_widget_show (vbox1);
-    gtk_container_add (GTK_CONTAINER (OptionDialog), vbox1);
-
     wPB_About = gtk_button_new ();
     gtk_widget_show (wPB_About);
     gtk_box_pack_start (GTK_BOX (vbox1), wPB_About, FALSE, FALSE, 0);
