@@ -24,8 +24,6 @@
  */
 #ifndef _config_gui_h
 #define _config_gui_h
-static char     _config_gui_h_id[] =
-    "$Id: config_gui.h,v 1.3 2004/08/25 10:08:40 rogerms Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -33,10 +31,8 @@ static char     _config_gui_h_id[] =
 
 #include <gtk/gtk.h>
 
-#include <libxfce4util/i18n.h>
+#include <libxfce4util/libxfce4util.h>
 #include <libxfcegui4/dialogs.h>
-#include <panel/plugins.h>
-#include <panel/xfce.h>
 
 
 typedef struct gui_t {
