@@ -23,16 +23,8 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-static char     _main_id[] =
-    "$Id: main.c,v 1.7 2004/08/25 10:08:40 rogerms Exp $";
-
-
-#define DEBUG	0
-
 #include "config_gui.h"
 #include "devperf.h"
-#include "debug.h"
-
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -40,10 +32,9 @@ static char     _main_id[] =
 
 #include <gtk/gtk.h>
 
-#include <libxfce4util/i18n.h>
-#include <libxfcegui4/dialogs.h>
-#include <panel/plugins.h>
-#include <panel/xfce.h>
+#include <libxfce4util/libxfce4util.h>
+#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4panel/xfce-panel-plugin.h>
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -57,6 +48,7 @@ static char     _main_id[] =
 
 
 #define PLUGIN_NAME	"DiskPerf"
+#define BORDER          8
 
 
  /* Some platforms do not provide busy times as separate read and write
@@ -139,16 +131,17 @@ typedef struct monitor_t {
                     oPrevPerf;
 } monitor_t;
 
-typedef struct plugin_t {
+typedef struct diskperf_t {
+    XfcePanelPlugin *plugin;
     unsigned int    iTimerId;	/* Cyclic update */
     struct conf_t   oConf;
     struct monitor_t
                     oMonitor;
-} plugin_t;
+} diskperf_t;
 
 	/**************************************************************/
 
-static int DisplayPerf (struct plugin_t *p_poPlugin)
+static int DisplayPerf (struct diskperf_t *p_poPlugin)
  /* Get the last disk perfomance data, compute the statistics and update
     the panel-docked monitor bars */
 {
@@ -275,27 +268,23 @@ static gboolean SetTimer (void *p_pvPlugin)
 	/* Recurrently update the panel-docked monitor bars through a
 	   timer */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_pvPlugin;
+    struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
 
-    if (poPlugin->iTimerId) {
-	g_source_remove (poPlugin->iTimerId);
-	poPlugin->iTimerId = 0;
-    }
-    XFCE_PANEL_LOCK ();
     DisplayPerf (poPlugin);
-    XFCE_PANEL_UNLOCK ();
-    poPlugin->iTimerId = g_timeout_add (poConf->iPeriod_ms,
-					(GtkFunction) SetTimer, poPlugin);
-    return (1);
+
+    if (!poPlugin->iTimerId)
+        poPlugin->iTimerId = g_timeout_add (poConf->iPeriod_ms,
+					    (GSourceFunc) SetTimer, poPlugin);
+    return TRUE;
 }				/* SetTimer() */
 
 	/**************************************************************/
 
-static int SetSingleBarColor (struct plugin_t *p_poPlugin, int p_iBar)
+static int SetSingleBarColor (struct diskperf_t *p_poPlugin, int p_iBar)
 	/* Set the color of a single monitor bar */
 {
-    struct plugin_t *poPlugin = p_poPlugin;
+    struct diskperf_t *poPlugin = p_poPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
     GtkRcStyle     *poStyle;
@@ -315,10 +304,10 @@ static int SetSingleBarColor (struct plugin_t *p_poPlugin, int p_iBar)
 
 	/**************************************************************/
 
-static int SetMonitorBarColor (struct plugin_t *p_poPlugin)
+static int SetMonitorBarColor (struct diskperf_t *p_poPlugin)
 	/* Set the monitor bar colors */
 {
-    struct plugin_t *poPlugin = p_poPlugin;
+    struct diskperf_t *poPlugin = p_poPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
 
     if (poConf->fRW_DataCombined)
@@ -332,10 +321,10 @@ static int SetMonitorBarColor (struct plugin_t *p_poPlugin)
 
 	/**************************************************************/
 
-static int ResetMonitorBar (struct plugin_t *p_poPlugin)
+static int ResetMonitorBar (struct diskperf_t *p_poPlugin)
 	/* Set order (Read-Write or Write-Read) and colors */
 {
-    struct plugin_t *poPlugin = p_poPlugin;
+    struct diskperf_t *poPlugin = p_poPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
 
@@ -352,23 +341,23 @@ static int ResetMonitorBar (struct plugin_t *p_poPlugin)
 
 	/**************************************************************/
 
-static int CreateMonitorBars (struct plugin_t *p_poPlugin,
-			      int p_iOrientation)
+static int CreateMonitorBars (struct diskperf_t *p_poPlugin,
+			      GtkOrientation p_iOrientation)
 	/* Create the panel progressive bars */
 {
-    struct plugin_t *poPlugin = p_poPlugin;
+    struct diskperf_t *poPlugin = p_poPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
     Widget_t       *pwBar;
     int             i;
 
-    if (p_iOrientation == HORIZONTAL)
+    if (p_iOrientation == GTK_ORIENTATION_HORIZONTAL)
 	poMonitor->wBox = gtk_hbox_new (FALSE, 0);
     else
 	poMonitor->wBox = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (poMonitor->wBox);
     gtk_container_set_border_width (GTK_CONTAINER
-				    (poMonitor->wBox), border_width);
+				    (poMonitor->wBox), 4);
 
     gtk_container_add (GTK_CONTAINER (poMonitor->wEventBox),
 		       poMonitor->wBox);
@@ -384,7 +373,7 @@ static int CreateMonitorBars (struct plugin_t *p_poPlugin,
 	*pwBar = GTK_WIDGET (gtk_progress_bar_new ());
 	gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR (*pwBar),
 					  (p_iOrientation ==
-					   HORIZONTAL ?
+					   GTK_ORIENTATION_HORIZONTAL ?
 					   GTK_PROGRESS_BOTTOM_TO_TOP :
 					   GTK_PROGRESS_LEFT_TO_RIGHT));
 	if ((i == 1) && poConf->fRW_DataCombined)
@@ -402,10 +391,11 @@ static int CreateMonitorBars (struct plugin_t *p_poPlugin,
 
 	/**************************************************************/
 
-static plugin_t *NewPlugin ()
-	/* New instance of the plugin (constructor) */
+static diskperf_t *diskperf_create_control (XfcePanelPlugin *plugin)
+	/* Plugin API */
+	/* Create the diskperf */
 {
-    struct plugin_t *poPlugin;
+    struct diskperf_t *poPlugin;
     struct param_t *poConf;
     struct monitor_t *poMonitor;
 #if !defined(__NetBSD__)
@@ -413,11 +403,13 @@ static plugin_t *NewPlugin ()
     int             status;
 #endif
 
-    poPlugin = g_new (plugin_t, 1);
-    memset (poPlugin, 0, sizeof (plugin_t));
+    poPlugin = g_new (diskperf_t, 1);
+    memset (poPlugin, 0, sizeof (diskperf_t));
     poConf = &(poPlugin->oConf.oParam);
     poMonitor = &(poPlugin->oMonitor);
 
+    poPlugin->plugin = plugin;
+    
 #if defined(__NetBSD__)
     strcpy (poConf->acDevice, "wd0");
     strcpy (poConf->acTitle, "wd0");
@@ -445,42 +437,20 @@ static plugin_t *NewPlugin ()
     poMonitor->wEventBox = gtk_event_box_new ();
     gtk_widget_show (poMonitor->wEventBox);
 
-    return (poPlugin);
-}				/* NewPlugin() */
+    xfce_panel_plugin_add_action_widget (plugin, poMonitor->wEventBox);
+    
+    return poPlugin;
+}				/* diskperf_create_control() */
 
 	/**************************************************************/
 
-static gboolean plugin_create_control (Control * p_poCtrl)
-	/* Plugin API */
-	/* Create the plugin */
-{
-    struct plugin_t *poPlugin;
-
-    TRACE ("plugin_create_control()\n");
-    poPlugin = NewPlugin ();
-    gtk_container_add (GTK_CONTAINER (p_poCtrl->base),
-		       poPlugin->oMonitor.wEventBox);
-    p_poCtrl->data = (gpointer) poPlugin;
-    p_poCtrl->with_popup = FALSE;
-    gtk_widget_set_size_request (p_poCtrl->base, -1, -1);
-    return (TRUE);
-}				/* plugin_create_control() */
-
-	/**************************************************************/
-
-static void plugin_free (Control * ctrl)
+static void diskperf_free (XfcePanelPlugin *plugin, diskperf_t *poPlugin)
 	/* Plugin API */
 {
-    struct plugin_t *poPlugin;
-
-    TRACE ("plugin_free()\n");
-    g_return_if_fail (ctrl != NULL);
-    g_return_if_fail (ctrl->data != NULL);
-    poPlugin = (plugin_t *) ctrl->data;
     if (poPlugin->iTimerId)
 	g_source_remove (poPlugin->iTimerId);
     g_free (poPlugin);
-}				/* plugin_free() */
+}				/* diskperf_free() */
 
 	/**************************************************************/
 
@@ -499,159 +469,148 @@ static void plugin_free (Control * ctrl)
 
 	/**************************************************************/
 
-static void plugin_read_config (Control * p_poCtrl, xmlNodePtr p_poParent)
+static void diskperf_read_config (XfcePanelPlugin *plugin, 
+                                  diskperf_t *poPlugin)
 	/* Plugin API */
 	/* Executed when the panel is started - Read the configuration
 	   previously stored in xml file */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_poCtrl->data;
+    const char *value;
+    char *file;
+    XfceRc *rc;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
     Widget_t       *pw2ndBar = poPlugin->oMonitor.awProgressBar + 1;
-    xmlNodePtr      poNode;
-    char           *pc;
 #if !defined(__NetBSD__)
     struct stat     oStat;
     int             status;
 #endif
+    
+    if (!(file = xfce_panel_plugin_lookup_rc_file (plugin)))
+        return;
+    
+    rc = xfce_rc_simple_open (file, TRUE);
+    g_free (file);
 
-    TRACE ("plugin_read_config()\n");
-    if (!p_poParent)
-	return;
-    for (poNode = p_poParent->children; poNode; poNode = poNode->next) {
-	if (!(xmlStrEqual (poNode->name, PLUGIN_NAME)))
-	    continue;
-
-	if ((pc = xmlGetProp (poNode, (CONF_DEVICE)))) {
-	    memset (poConf->acDevice, 0, sizeof (poConf->acDevice));
-	    strncpy (poConf->acDevice, pc, sizeof (poConf->acDevice) - 1);
-	    xmlFree (pc);
+    if (!rc)
+        return;
+    
+    if ((value = xfce_rc_read_entry (rc, (CONF_DEVICE), NULL))) {
+        memset (poConf->acDevice, 0, sizeof (poConf->acDevice));
+        strncpy (poConf->acDevice, value, sizeof (poConf->acDevice) - 1);
 #if !defined(__NetBSD__)
-	    status = stat (poConf->acDevice, &oStat);
-	    poConf->st_rdev = (status == -1 ? 0 : oStat.st_rdev);
+        status = stat (poConf->acDevice, &oStat);
+        poConf->st_rdev = (status == -1 ? 0 : oStat.st_rdev);
 #endif
-	}
-
-	if ((pc = xmlGetProp (poNode, (CONF_USE_LABEL)))) {
-	    poConf->fTitleDisplayed = atoi (pc);
-	    xmlFree (pc);
-	}
-	if (poConf->fTitleDisplayed)
-	    gtk_widget_show (GTK_WIDGET (poMonitor->wTitle));
-	else
-	    gtk_widget_hide (GTK_WIDGET (poMonitor->wTitle));
-
-	if ((pc = xmlGetProp (poNode, (CONF_LABEL_TEXT)))) {
-	    memset (poConf->acTitle, 0, sizeof (poConf->acTitle));
-	    strncpy (poConf->acTitle, pc, sizeof (poConf->acTitle) - 1);
-	    xmlFree (pc);
-	    gtk_label_set_text (GTK_LABEL (poMonitor->wTitle),
-				poConf->acTitle);
-	}
-
-	if ((pc = xmlGetProp (poNode, (CONF_UPDATE_PERIOD)))) {
-	    poConf->iPeriod_ms = atoi (pc);
-	    xmlFree (pc);
-	}
-
-	if ((pc = xmlGetProp (poNode, (CONF_STATISTICS)))) {
-	    poConf->eStatistics = atoi (pc);
-	    xmlFree (pc);
-	}
-
-	if ((pc = xmlGetProp (poNode, (CONF_XFER_RATE)))) {
-	    poConf->iMaxXferMBperSec = atoi (pc);
-	    xmlFree (pc);
-	}
-
-	if ((pc = xmlGetProp (poNode, (CONF_COMBINE_RW_DATA)))) {
-	    poConf->fRW_DataCombined = atoi (pc);
-	    xmlFree (pc);
-	}
-	if (poConf->fRW_DataCombined)
-	    gtk_widget_hide (GTK_WIDGET (*pw2ndBar));
-	else
-	    gtk_widget_show (GTK_WIDGET (*pw2ndBar));
-
-	if ((pc = xmlGetProp (poNode, (CONF_MONITOR_BAR_ORDER)))) {
-	    poConf->eMonitorBarOrder = atoi (pc);
-	    xmlFree (pc);
-	}
-
-	if ((pc = xmlGetProp (poNode, (CONF_READ_COLOR)))) {
-	    gdk_color_parse (pc, poConf->aoColor + R_DATA);
-	    xmlFree (pc);
-	}
-
-	if ((pc = xmlGetProp (poNode, (CONF_WRITE_COLOR)))) {
-	    gdk_color_parse (pc, poConf->aoColor + W_DATA);
-	    xmlFree (pc);
-	}
-
-	if ((pc = xmlGetProp (poNode, (CONF_READ_WRITE_COLOR)))) {
-	    gdk_color_parse (pc, poConf->aoColor + RW_DATA);
-	    xmlFree (pc);
-	}
-
-	ResetMonitorBar (poPlugin);
     }
-    SetTimer (poPlugin);
-}				/* plugin_read_config() */
+
+    poConf->fTitleDisplayed = 
+        xfce_rc_read_int_entry (rc, (CONF_USE_LABEL), 1);
+
+    if (poConf->fTitleDisplayed)
+        gtk_widget_show (GTK_WIDGET (poMonitor->wTitle));
+    else
+        gtk_widget_hide (GTK_WIDGET (poMonitor->wTitle));
+
+    if ((value = xfce_rc_read_entry (rc, (CONF_LABEL_TEXT), NULL))) {
+        memset (poConf->acTitle, 0, sizeof (poConf->acTitle));
+        strncpy (poConf->acTitle, value, sizeof (poConf->acTitle) - 1);
+        gtk_label_set_text (GTK_LABEL (poMonitor->wTitle),
+                            poConf->acTitle);
+    }
+
+    poConf->iPeriod_ms = 
+        xfce_rc_read_int_entry (rc, (CONF_UPDATE_PERIOD), 500);
+    poConf->eStatistics = 
+        xfce_rc_read_int_entry (rc, (CONF_STATISTICS), IO_TRANSFER);
+
+    poConf->iMaxXferMBperSec = 
+        xfce_rc_read_int_entry (rc, (CONF_XFER_RATE), 40);
+
+    poConf->fRW_DataCombined = 
+        xfce_rc_read_int_entry (rc, (CONF_COMBINE_RW_DATA), 1);
+
+    if (poConf->fRW_DataCombined)
+        gtk_widget_hide (GTK_WIDGET (*pw2ndBar));
+    else
+        gtk_widget_show (GTK_WIDGET (*pw2ndBar));
+
+    poConf->eMonitorBarOrder = 
+        xfce_rc_read_int_entry (rc, (CONF_MONITOR_BAR_ORDER), RW_ORDER);
+
+    if ((value = xfce_rc_read_entry (rc, (CONF_READ_COLOR), NULL))) {
+        gdk_color_parse (value, poConf->aoColor + R_DATA);
+    }
+
+    if ((value = xfce_rc_read_entry (rc, (CONF_WRITE_COLOR), NULL))) {
+        gdk_color_parse (value, poConf->aoColor + W_DATA);
+    }
+
+    if ((value = xfce_rc_read_entry (rc, (CONF_READ_WRITE_COLOR), NULL))) {
+        gdk_color_parse (value, poConf->aoColor + RW_DATA);
+    }
+
+    xfce_rc_close (rc);
+}				/* diskperf_read_config() */
 
 	/**************************************************************/
 
-static void plugin_write_config (Control * p_poCtrl, xmlNodePtr p_poParent)
+static void diskperf_write_config (XfcePanelPlugin *plugin, 
+                                   diskperf_t *poPlugin)
 	/* Plugin API */
-	/* Write plugin configuration into xml file */
+	/* Write diskperf configuration into xml file */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_poCtrl->data;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     const char     *acColorFormat = "#%02X%02X%02X";
-    xmlNodePtr      poRoot;
     GdkColor       *poColor;
     char            acBuffer[16];
+    XfceRc *rc;
+    char *file;
 
-    TRACE ("plugin_write_config()\n");
+    if (!(file = xfce_panel_plugin_save_location (plugin, TRUE)))
+        return;
+    
+    rc = xfce_rc_simple_open (file, FALSE);
+    g_free (file);
 
-    poRoot = xmlNewTextChild (p_poParent, NULL, PLUGIN_NAME, NULL);
+    if (!rc)
+        return;
+    
+    xfce_rc_write_entry (rc, CONF_DEVICE, poConf->acDevice);
 
-    xmlSetProp (poRoot, CONF_DEVICE, poConf->acDevice);
+    xfce_rc_write_int_entry (rc, CONF_USE_LABEL, poConf->fTitleDisplayed);
 
-    sprintf (acBuffer, "%d", poConf->fTitleDisplayed);
-    xmlSetProp (poRoot, CONF_USE_LABEL, acBuffer);
+    xfce_rc_write_entry (rc, CONF_LABEL_TEXT, poConf->acTitle);
 
-    xmlSetProp (poRoot, CONF_LABEL_TEXT, poConf->acTitle);
+    xfce_rc_write_int_entry (rc, CONF_UPDATE_PERIOD, poConf->iPeriod_ms);
 
-    sprintf (acBuffer, "%d", poConf->iPeriod_ms);
-    xmlSetProp (poRoot, CONF_UPDATE_PERIOD, acBuffer);
+    xfce_rc_write_int_entry (rc, CONF_STATISTICS, poConf->eStatistics);
 
-    sprintf (acBuffer, "%d", poConf->eStatistics);
-    xmlSetProp (poRoot, CONF_STATISTICS, acBuffer);
+    xfce_rc_write_int_entry (rc, CONF_XFER_RATE, poConf->iMaxXferMBperSec);
 
-    sprintf (acBuffer, "%d", poConf->iMaxXferMBperSec);
-    xmlSetProp (poRoot, CONF_XFER_RATE, acBuffer);
+    xfce_rc_write_int_entry (rc, CONF_COMBINE_RW_DATA, 
+                             poConf->fRW_DataCombined);
 
-    sprintf (acBuffer, "%d", poConf->fRW_DataCombined);
-    xmlSetProp (poRoot, CONF_COMBINE_RW_DATA, acBuffer);
-
-    sprintf (acBuffer, "%d", poConf->eMonitorBarOrder);
-    xmlSetProp (poRoot, CONF_MONITOR_BAR_ORDER, acBuffer);
+    xfce_rc_write_int_entry (rc, CONF_MONITOR_BAR_ORDER, 
+                             poConf->eMonitorBarOrder);
 
     poColor = poConf->aoColor + R_DATA;
     sprintf (acBuffer, acColorFormat,
 	     poColor->red >> 8, poColor->green >> 8, poColor->blue >> 8);
-    xmlSetProp (poRoot, CONF_READ_COLOR, acBuffer);
+    xfce_rc_write_entry (rc, CONF_READ_COLOR, acBuffer);
 
     poColor = poConf->aoColor + W_DATA;
     sprintf (acBuffer, acColorFormat,
 	     poColor->red >> 8, poColor->green >> 8, poColor->blue >> 8);
-    xmlSetProp (poRoot, CONF_WRITE_COLOR, acBuffer);
+    xfce_rc_write_entry (rc, CONF_WRITE_COLOR, acBuffer);
 
     poColor = poConf->aoColor + RW_DATA;
     sprintf (acBuffer, acColorFormat,
 	     poColor->red >> 8, poColor->green >> 8, poColor->blue >> 8);
-    xmlSetProp (poRoot, CONF_READ_WRITE_COLOR, acBuffer);
-}				/* plugin_write_config() */
+    xfce_rc_write_entry (rc, CONF_READ_WRITE_COLOR, acBuffer);
+
+    xfce_rc_close (rc);
+}				/* diskperf_write_config() */
 
 	/**************************************************************/
 
@@ -659,7 +618,7 @@ static void SetDevice (Widget_t p_wTF, void *p_pvPlugin)
 	/* GUI callback setting the device name, whose performance will be 
 	   displayed using the panel-docked monitor bars */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_pvPlugin;
+    struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     const char     *pcDevice = gtk_entry_get_text (GTK_ENTRY (p_wTF));
 #if !defined(__NetBSD__)
@@ -684,7 +643,7 @@ static void SetDevice (Widget_t p_wTF, void *p_pvPlugin)
 static void ToggleTitle (Widget_t p_w, void *p_pvPlugin)
 	/* GUI callback turning on/off the monitor bar legend */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_pvPlugin;
+    struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct gui_t   *poGUI = &(poPlugin->oConf.oGUI);
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
@@ -704,7 +663,7 @@ static void ToggleTitle (Widget_t p_w, void *p_pvPlugin)
 static void SetLabel (Widget_t p_wTF, void *p_pvPlugin)
 	/* GUI callback setting the legend of the monitor bars */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_pvPlugin;
+    struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
     const char     *acTitle = gtk_entry_get_text (GTK_ENTRY (p_wTF));
@@ -719,7 +678,7 @@ static void SetLabel (Widget_t p_wTF, void *p_pvPlugin)
 static void ToggleStatistics (Widget_t p_w, void *p_pvPlugin)
 	/* GUI callback allowing to choose statistics to monitor */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_pvPlugin;
+    struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct gui_t   *poGUI = &(poPlugin->oConf.oGUI);
 
@@ -751,7 +710,7 @@ static void ToggleRWintegration (Widget_t p_w, void *p_pvPlugin)
 	   single monitor bar, or to keep them separate using 2 dedicated
 	   monitor bars */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_pvPlugin;
+    struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct gui_t   *poGUI = &(poPlugin->oConf.oGUI);
     Widget_t       *pw2ndBar = poPlugin->oMonitor.awProgressBar + 1;
@@ -777,7 +736,7 @@ static void ToggleRWintegration (Widget_t p_w, void *p_pvPlugin)
 static void ToggleRWorder (Widget_t p_w, void *p_pvPlugin)
 	/* GUI callback allowing to swap Read/Write monitor bars */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_pvPlugin;
+    struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
 
     poConf->eMonitorBarOrder = !(poConf->eMonitorBarOrder);
@@ -791,7 +750,7 @@ static void SetXferRate (Widget_t p_wTF, void *p_pvPlugin)
 	/* GUI callback setting the maximum I/O transfer rate of the
 	   device */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_pvPlugin;
+    struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     const char     *pcXferRate = gtk_entry_get_text (GTK_ENTRY (p_wTF));
     float           r;
@@ -810,7 +769,7 @@ static void SetXferRate (Widget_t p_wTF, void *p_pvPlugin)
 static void SetPeriod (Widget_t p_wSc, void *p_pvPlugin)
 	/* Set the update period - To be used by the timer */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_pvPlugin;
+    struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     float           r;
 
@@ -828,7 +787,7 @@ static void SetPeriod (Widget_t p_wSc, void *p_pvPlugin)
 
 static void ChooseColor (Widget_t p_wPB, void *p_pvPlugin)
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_pvPlugin;
+    struct diskperf_t *poPlugin = (diskperf_t *) p_pvPlugin;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct gui_t   *poGUI = &(poPlugin->oConf.oGUI);
     Widget_t        wDialog;
@@ -869,10 +828,9 @@ static void ChooseColor (Widget_t p_wPB, void *p_pvPlugin)
 
 	/**************************************************************/
 
-static void UpdateConf (Widget_t w, void *p_pvPlugin)
+static void UpdateConf (diskperf_t *poPlugin)
 	/* Called back when the configuration/options window is closed */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_pvPlugin;
     struct conf_t  *poConf = &(poPlugin->oConf);
     struct gui_t   *poGUI = &(poConf->oGUI);
 
@@ -897,26 +855,26 @@ static int CheckStatsAvailability ()
 	return (0);
     if (status < 0) {
 	status *= -1;
-	xfce_err ("%s\n"
+	xfce_err (_("%s\n"
 		  "%s: %s (%d)\n\n"
 		  "This monitor will not work!\n"
-		  "Please remove it.",
+		  "Please remove it."),
 		  PLUGIN_NAME, (pcStatFile ? pcStatFile : ""),
 		  strerror (status), status);
 	return (-1);
     }
     switch (status) {
 	case NO_EXTENDED_STATS:
-	    xfce_err ("%s: No disk extended statistics found!\n"
+	    xfce_err (_("%s: No disk extended statistics found!\n"
 		      "Either old kernel (< 2.4.20) or not\n"
 		      "compiled with CONFIG_BLK_STATS turned on.\n\n"
 		      "This monitor will not work!\n"
-		      "Please remove it.", PLUGIN_NAME);
+		      "Please remove it."), PLUGIN_NAME);
 	    break;
 	default:
-	    xfce_err ("%s: Unknown error\n\n"
+	    xfce_err (_("%s: Unknown error\n\n"
 		      "This monitor will not work!\n"
-		      "Please remove it.", PLUGIN_NAME);
+		      "Please remove it."), PLUGIN_NAME);
     }
     return (-1);
 }				/* CheckStatsAvailability() */
@@ -926,24 +884,32 @@ static int CheckStatsAvailability ()
 static void About (Widget_t w, void *unused)
 	/* Called back when the About button in clicked */
 {
-    xfce_info ("%s %s - Disk Performance Monitor\n"
+    xfce_info (_("%s %s - Disk Performance Monitor\n"
 	       "Display instantaneous disk I/O transfer rates and busy times "
 	       "on Linux and NetBSD systems\n\n"
 	       "(c) 2003, 2004 Roger Seguin <roger_seguin@msn.com>\n"
 	       "NetBSD statistics collection: (c) 2003 Benedikt Meurer\n"
-	       "\t<benedikt.meurer@unix-ag.uni-siegen.de>",
+	       "\t<benedikt.meurer@unix-ag.uni-siegen.de>"),
 	       PACKAGE, VERSION);
 }				/* About() */
 
 	/**************************************************************/
 
-static void plugin_create_options (Control * p_poCtrl,
-				   GtkContainer * p_pxContainer,
-				   Widget_t p_wDone)
+static void diskperf_dialog_response (GtkWidget *dlg, int response, 
+                                      diskperf_t *diskperf)
+{
+    UpdateConf (diskperf);
+    gtk_widget_destroy (dlg);
+    xfce_panel_plugin_unblock_menu (diskperf->plugin);
+    diskperf_write_config (diskperf->plugin, diskperf);
+}
+
+static void diskperf_create_options (XfcePanelPlugin *plugin,
+                                     diskperf_t *poPlugin)
 	/* Plugin API */
 	/* Create/pop up the configuration/options GUI */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_poCtrl->data;
+    GtkWidget *dlg, *header, *vbox;
     struct param_t *poConf = &(poPlugin->oConf.oParam);
     struct gui_t   *poGUI = &(poPlugin->oConf.oGUI);
     char            acBuffer[16];
@@ -951,13 +917,40 @@ static void plugin_create_options (Control * p_poCtrl,
     Widget_t       *apwColorPB[NMONITORS];
     int             i;
 
-    TRACE ("plugin_create_options()\n");
+    TRACE ("diskperf_create_options()\n");
 
     (void) CheckStatsAvailability ();
 
-    poPlugin->oConf.wTopLevel = gtk_widget_get_toplevel (p_wDone);
+    xfce_panel_plugin_block_menu (plugin);
+    
+    dlg = gtk_dialog_new_with_buttons (_("Configuration"), 
+                GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
+                GTK_DIALOG_DESTROY_WITH_PARENT |
+                GTK_DIALOG_NO_SEPARATOR,
+                GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
+                NULL);
+    
+    g_signal_connect (dlg, "response", G_CALLBACK (diskperf_dialog_response),
+                      poPlugin);
 
-    (void) CreateConfigGUI (GTK_WIDGET (p_pxContainer), poGUI);
+    gtk_container_set_border_width (GTK_CONTAINER (dlg), 2);
+    
+    header = xfce_create_header (NULL, _("Disk Performance Monitor"));
+    gtk_widget_set_size_request (GTK_BIN (header)->child, -1, 32);
+    gtk_container_set_border_width (GTK_CONTAINER (header), BORDER - 2);
+    gtk_widget_show (header);
+    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), header,
+                        FALSE, TRUE, 0);
+    
+    vbox = gtk_vbox_new(FALSE, BORDER);
+    gtk_container_set_border_width (GTK_CONTAINER (vbox), BORDER - 2);
+    gtk_widget_show(vbox);
+    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), vbox,
+                        TRUE, TRUE, 0);
+    
+    poPlugin->oConf.wTopLevel = dlg;
+
+    (void) CreateConfigGUI (vbox, poGUI);
 
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (poGUI->wTB_Title),
 				  poConf->fTitleDisplayed);
@@ -1041,60 +1034,52 @@ static void plugin_create_options (Control * p_poCtrl,
 			  G_CALLBACK (ChooseColor), poPlugin);
     }
 
-    g_signal_connect (GTK_WIDGET (p_wDone), "clicked",
-		      G_CALLBACK (UpdateConf), poPlugin);
-		      
     g_signal_connect (GTK_WIDGET (poGUI->wPB_About), "clicked",
 		      G_CALLBACK (About), 0);
 		      
 		      
-
-}				/* plugin_create_options() */
+    gtk_widget_show (dlg);
+}				/* diskperf_create_options() */
 
 	/**************************************************************/
 
-static void plugin_attach_callback (Control * ctrl, const gchar * signal,
-				    GCallback cb, gpointer data)
-	/* Plugin API */
-	/* This function has to be defined, even when not being used at
-	   all */
-{
-    TRACE ("plugin_attach_callback()\n");
-}				/* plugin_attach_callback() */
-
-	/**************************************************************/
-
-static void plugin_set_size (Control * p_poCtrl, int p_size)
+static gboolean diskperf_set_size (XfcePanelPlugin *plugin, int p_size,
+                               diskperf_t *poPlugin)
 	/* Plugin API */
 	/* Set the size of the panel-docked monitor bars */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_poCtrl->data;
-    int             size1, size2;
-    int             i;
+    int             i, size1, size2;
     Widget_t       *pwBar;
 
-    TRACE ("plugin_set_size()\n");
-    if (settings.orientation == HORIZONTAL)
-	size1 = 6 + 2 * p_size, size2 = icon_size[p_size];
-    else
-	size1 = icon_size[p_size], size2 = 6 + 2 * p_size;
+    TRACE ("diskperf_set_size()\n");
+    if (xfce_panel_plugin_get_orientation (plugin) == 
+            GTK_ORIENTATION_HORIZONTAL) {
+	size1 = BORDER;
+        size2 = p_size - 4;
+    }
+    else {
+	size1 = p_size - 4;
+        size2 = BORDER;
+    }
     for (i = 0; i < 2; i++) {
 	pwBar = poPlugin->oMonitor.awProgressBar + i;
 	gtk_widget_set_size_request (GTK_WIDGET (*pwBar), size1, size2);
-	gtk_widget_queue_resize (GTK_WIDGET (*pwBar));
     }
-}				/* plugin_set_size() */
+
+    return TRUE;
+}				/* diskperf_set_size() */
 
 	/**************************************************************/
 
-static void plugin_set_orientation (Control * p_poCtrl, int p_iOrientation)
+static void diskperf_set_orientation (XfcePanelPlugin *plugin, 
+                                      GtkOrientation p_iOrientation,
+                                      diskperf_t *poPlugin)
 	/* Plugin API */
 	/* Invoked when the panel changes orientation */
 {
-    struct plugin_t *poPlugin = (plugin_t *) p_poCtrl->data;
     struct monitor_t *poMonitor = &(poPlugin->oMonitor);
 
-    TRACE ("plugin_set_orientation()\n");
+    TRACE ("diskperf_set_orientation()\n");
     if (poPlugin->iTimerId) {
 	g_source_remove (poPlugin->iTimerId);
 	poPlugin->iTimerId = 0;
@@ -1103,33 +1088,45 @@ static void plugin_set_orientation (Control * p_poCtrl, int p_iOrientation)
 			  GTK_WIDGET (poMonitor->wBox));
     CreateMonitorBars (poPlugin, p_iOrientation);
     SetTimer (poPlugin);
-}				/* plugin_set_orientation() */
+}				/* diskperf_set_orientation() */
 
 	/**************************************************************/
 
-G_MODULE_EXPORT void xfce_control_class_init (ControlClass * cc)
-	/* Plugin API */
-	/* Main */
+static void diskperf_construct (XfcePanelPlugin *plugin)
 {
-    TRACE ("xfce_control_class_init()\n");
+    diskperf_t *diskperf = diskperf_create_control (plugin);
 
-    (void) DevPerfInit ();
+    diskperf_read_config (plugin, diskperf);
+
+    g_signal_connect (plugin, "free-data", G_CALLBACK (diskperf_free), 
+                      diskperf);
+
+    g_signal_connect (plugin, "save", G_CALLBACK (diskperf_write_config), 
+                      diskperf);
+
+    g_signal_connect (plugin, "size-changed", G_CALLBACK (diskperf_set_size), 
+                      diskperf);
+
+    g_signal_connect (plugin, "orientation-changed", 
+                      G_CALLBACK (diskperf_set_orientation), diskperf);
+
+    xfce_panel_plugin_menu_show_about (plugin);
+    g_signal_connect (plugin, "about", G_CALLBACK (About), NULL);
+
+    xfce_panel_plugin_menu_show_configure (plugin);
+    g_signal_connect (plugin, "configure-plugin", 
+                      G_CALLBACK (diskperf_create_options), diskperf);
+
+    gtk_container_add (GTK_CONTAINER (plugin), diskperf->oMonitor.wEventBox);
+
+    CreateMonitorBars (diskperf, xfce_panel_plugin_get_orientation (plugin));
+    
+    SetTimer (diskperf);
+}
+
+XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL (diskperf_construct);
 
-    cc->name = PLUGIN_NAME;
-    cc->caption = _("Disk Performance");
-    cc->create_control = (CreateControlFunc) plugin_create_control;
-    cc->free = plugin_free;
-    cc->attach_callback = plugin_attach_callback;
-    cc->read_config = plugin_read_config;
-    cc->write_config = plugin_write_config;
-    cc->create_options = plugin_create_options;
-    cc->set_size = plugin_set_size;
-    cc->set_orientation = plugin_set_orientation;
-    cc->set_theme = 0;
-}				/* xfce_control_class_init() */
 
-	/* Plugin API */
-XFCE_PLUGIN_CHECK_INIT
 	/**************************************************************/
 /*
 $Log: main.c,v $
